#include "rjte.h"

/* Set this when 20 seconds have elapsed and we need to go to state 4 */
volatile u8 stop;

/* Timer 0 overflow interrupt handler */
SIGNAL(SIG_OVERFLOW0)
{
	static u16 count;

	/* 56.25 overflows/second * 20 seconds == 1125 overflows */
	if (count < 1125)
		count++;
	else stop = 1;
}

int main()
{
	u8 state, count;
	u8 cur_button_states, last_button_states, button_press;

	/* Hardware initialization, must call this first */
	initialize();

	/* Set up timer 0 */
	TCCR0 = 5;		/* clk/1024 prescalar, overflows at 56.25Hz */

	/* Turn on the overflow interrupt for timer 0 */
	TIMSK |= (1 << TOIE0);

	/* Initially in state 1 */
	state = 1;

	/* Number of button presses */
	count = 0;

	/* Used to detect button presses */
	last_button_states = 0;

	while (1)
	{
		u8 left_button, right_button;

		/* Button press logic */

		/* Read this once in case it changes during the code later.
		 * After this line, don't read button_states again because it
		 * may be different.
		 */
		cur_button_states = button_states;

		/* Determine which buttons have been pressed.
		 * Each bit in button_press will be set if the corresponding button
		 * is pressed on this iteration but wasn't on the last iteration.
		 */
		button_press = cur_button_states & ~last_button_states;

		/* Save button states for the next iteration */
		last_button_states = cur_button_states;

		/* This is only done for readability.  It would be more efficient to
		 * use the bitfields in button_press directly, perhaps with defines.
		 */
		left_button = button_press & 1;
		right_button = button_press & 2;

		/********
		 * State machine starts here
		 ********/

		/* Count button presses */
		if (left_button || right_button)
		{
			/* Got another button press */
			count++;
			if (count >= 20)
				state = 4;
		}

		/* Check if too much time has elapsed */
		if (stop)
			state = 4;

		/* Use two separate switch statements so the change becomes visible immediately.
		 * Otherwise the LEDs would be set according to the old state and the new state won't
		 * be seen for another 500ms.
		 */
		switch (state)
		{
			case 1:
				if (left_button)
					state = 3; 
				else if (right_button)
					state = 2;
				break;
			case 2:
				if (left_button)
					state = 1;
				else if (right_button)
					state = 3;
				break;
			case 3:
				if (left_button)
					state = 2;
				else if (right_button)
					state = 1;
				break;
			default:
				break;
		}

		/* Set the LEDs according to the (new) current state. */
		switch (state)
		{
			case 1:
				LED1 = ON;
				LED2 = OFF;
				LED3 = OFF;
				break;
			case 2:
				LED1 = OFF;
				LED2 = ON;
				LED3 = OFF;
				break;
			case 3:
				LED1 = OFF;
				LED2 = OFF;
				LED3 = ON;
				break;
			case 4:
				LED1 = ON;
				LED2 = ON;
				LED3 = ON;
				break;
			default:
				break;
		}
	}

	return 0;
}

